@@ -26,6 +26,10 @@
 
 
 
+(defparameter *DEFAULT-PATHNAME-DEFAULTS* #p".")
+
+
+
 (defun machine-instance nil)
 (defun machine-type nil)
 (defun machine-version nil)
@@ -71,6 +75,8 @@
 
 (defun identity (x) x)
 
+(defun constantly (x) (lambda (&rest r) x))
+
 
 
 (defparameter *READ-DEFAULT-FLOAT-FORMAT* 'single-float)
@@ -241,21 +247,22 @@
     (t (+ ""))))
 
 
-
+#|
 (defparameter *gensym-counter* 1)
 
 (defun gensym (&optional x)
   (let ((str (make-string-output-stream)))
     (cond
-      ((stringp x) (write-string x str)
+      ((typep x 'string) (write-string x str)
        (write *gensym-counter* :stream str)
        (incf *gensym-counter*))
-      ((integerp x) (write-string "G" str)
+      ((typep x 'integer) (write-string "G" str)
        (write x :stream str))
       ((null x) (write-string "G" str)
        (write *gensym-counter* :stream str)
        (incf *gensym-counter*)))
     (make-symbol (get-output-stream-string str))))
+|#
 
 
 (defparameter gentemp-counter 1)
@@ -552,6 +559,32 @@
 
 
 
+(defmacro defsetf (accessfn secondarg &rest args)
+  (if (listp secondarg)
+      (let (formsymsyms storesymsyms)
+	(dolist (arg secondarg)
+	  (setq formsymsyms (cons (gensym) formsymsyms)))
+	(setq formsymsyms (reverse formsymsyms))
+	(dolist (storevar (car args))
+	  (setq storesymsyms (cons (gensym) storesymsyms)))
+	(setq storesymsyms (reverse storesymsyms))
+	`(define-setf-expander ,accessfn ,secondarg
+	   (let (,@(mapcar (lambda (s) (list s '(gensym))) formsymsyms)
+		 ,@(mapcar (lambda (s) (list s '(gensym))) storesymsyms))
+	     (values
+	      (list ,@formsymsyms)
+	      (list ,@secondarg)
+	      (list ,@storesymsyms)
+	      (block ,accessfn
+		(let (,@(mapcar (lambda (s v) (list s v)) secondarg formsymsyms)
+		      ,@(mapcar (lambda (s v) (list s v)) (car args) storesymsyms))
+		  ,@(cdr args)))
+	      (list ',accessfn ,@formsymsyms)))))
+      (progn
+	)))
+
+
+
 (defmacro when (clause &body body)
   `(if ,clause (progn ,@body)))
 
@@ -560,42 +593,63 @@
   `(if (not ,clause) (progn ,@body)))
 
 
-(defmacro incf (place &optional (delta 1))
-  `(setf ,place (+ ,place ,delta)))
+(defmacro define-modify-macro (name lambda-list func &optional doc)
+  (let (args)
+    (dolist (l lambda-list)
+      (unless (eq l '&optional)
+	(if (typep l 'cons)
+	    (setq args (cons (car l) args))
+	    (setq args (cons l args)))))
+    `(defmacro ,name (place . ,lambda-list)
+       (let ((exp (multiple-value-list (get-setf-expansion place)))
+	      letf)
+	  (dolist (f (car exp))
+	    (setq letf (cons (list f (caadr exp)) letf))
+	    (setf (cadr exp) (cdadr exp)))
+	  (setq letf (cons (list (caaddr exp) (list ',func (nth 4 exp) ,@args)) letf))
+	  (setq letf (list 'let* (reverse letf) nil))
+	  (setf (caddr letf) (nth 3 exp))
+	  letf))))
 
 
-(defmacro decf (place &optional (delta 1))
-  `(setf ,place (- ,place ,delta)))
+(define-modify-macro incf (&optional (delta 1)) +)
+
+(define-modify-macro decf (&optional (delta 1)) -)
 
 
 (defmacro and (&rest forms)
-  `(let ((val (multiple-value-list ,(car forms))))
-     (if ,(not forms)
-         t
-         (if (car val)
-             (if ,(not (cdr forms))
-                 (values-list val)
-                 (and ,@(cdr forms)))))))
+  (if (not forms)
+      t
+      (let ((valsym (gensym)))
+	`(let ((,valsym (multiple-value-list ,(car forms))))
+           (if (car ,valsym)
+               ,(if (not (cdr forms))
+                    `(values-list ,valsym)
+                    `(and ,@(cdr forms))))))))
 
 
 (defmacro or (&rest forms)
-  `(let ((val (multiple-value-list ,(car forms))))
-     (if ,(not forms)
-       nil
-       (if (car val)
-	   (if ,(not (cdr forms))
-               (values-list val)
-	       (car val))
-           (or ,@(cdr forms))))))
+  (if (not forms)
+      nil
+      (let ((valsym (gensym)))
+	`(let ((,valsym (multiple-value-list ,(car forms))))
+	   (if (car ,valsym)
+	       ,(if (not (cdr forms))
+		    `(values-list ,valsym)
+		    `(car ,valsym))
+               (or ,@(cdr forms)))))))
 
 
 (defmacro cond (&body body)
-  (let ((first (car body))
-	(rest (cdr body)))
-    `(if ,(car first)
-	 (progn ,@(cdr first))
-	 (if ',rest
-	     (cond ,@rest)))))
+  (when body
+    (let ((first (car body))
+	  (rest (cdr body))
+	  (valsym (gensym)))
+      `(let ((,valsym ,(car first)))
+	 (if ,valsym
+	     (progn ,valsym ,@(cdr first))
+	     (cond ,@rest))))))
+
 
 
 (defmacro case (keyf &rest clauses)
@@ -606,7 +660,7 @@
 	  (cond ((or (eq (car clause) t)
 		     (eq (car clause) 'otherwise))
 		 (setq outsym (append outsym `((t (progn ,@(cdr clause)))))))
-		((listp (car clause))
+		((typep (car clause) 'list)
 		 (setq outsym (append outsym `(((member ,keysym ',(car clause))
 						(progn ,@(cdr clause)))))))
 		(t
@@ -1057,13 +1111,58 @@
 
 
 (defmacro push (item place)
-  `(setf ,place (cons ,item ,place)))
+  (let ((exp (multiple-value-list (get-setf-expansion place)))
+	(itemkey (gensym))
+	letf)
+    (dolist (f (car exp))
+      (setq letf (cons (cons f (cadr exp)) letf))
+      (setf (cadr exp) (cdadr exp)))
+    (setq letf (reverse letf))
+    `(let* ((,itemkey ,item)
+	    ,@letf
+	    (,(caaddr exp) (cons ,itemkey ,(nth 4 exp))))
+       ,(nth 3 exp)
+       ,(nth 4 exp))))
+
+
+(defmacro pushnew (item place &key key test test-not)
+  (let ((exp (multiple-value-list (get-setf-expansion place)))
+	(blocksym (gensym))
+	(itemsym (gensym))
+	(keysym (gensym))
+	(tstsym (gensym))
+	letf)
+    (dolist (f (car exp))
+      (setq letf (cons (cons f (cadr exp)) letf))
+      (setf (cadr exp) (cdadr exp)))
+    (setq letf (reverse letf))
+    `(block ,blocksym
+       (let* ((,itemsym ,item)
+	      ,@letf
+	      (,keysym (or ,key #'identity))
+	      (,tstsym (or ,test
+			   (if ,test-not (complement ,test-not))
+			   #'eql))
+	      (,(caaddr exp) (cons ,itemsym ,(nth 4 exp))))
+	 (dolist (l ,(nth 4 exp))
+	   (if (funcall ,tstsym (car ,(caaddr exp)) (funcall ,keysym l))
+	       (return-from ,blocksym ,(nth 4 exp))))
+	 ,(nth 3 exp)))))
 
 
 (defmacro pop (place)
-  `(let ((c (car ,place)))
-     (setf ,place (cdr ,place))
-     c))
+  (let ((exp (multiple-value-list (get-setf-expansion place)))
+	(carsym (gensym))
+	letf)
+    (dolist (f (car exp))
+      (setq letf (cons (cons f (cadr exp)) letf))
+      (setf (cadr exp) (cdadr exp)))
+    (setq letf (reverse letf))
+    `(let* (,@letf
+	    (,carsym (car ,(nth 4 exp)))
+	    (,(caaddr exp) (cdr ,(nth 4 exp))))
+       ,(nth 3 exp)
+       ,carsym)))
 
 
 (defun set-difference (l1 l2 &key key test test-not)
@@ -1231,6 +1330,13 @@
 	  (array-dimensions array))))
 
 
+(defun array-element-type (array)
+  (cond
+    ((stringp array) 'character)
+    ((bit-vector-p array) 'bit)
+    (t)))
+
+
 (defun upgraded-array-element-type (ts &optional env)
   (cond
     ((subtypep ts 'bit) 'bit)
@@ -1315,6 +1421,22 @@
   (setf (aref vec ind) newval))
 
 
+(defun vector-pop (vec)
+  (setf (fill-pointer vec) (1- (fill-pointer vec)))
+  (aref vec (fill-pointer vec)))
+
+(defun vector-push (newel vec)
+  (unless (>= (fill-pointer vec) (array-dimension vec 0))
+    (setf (aref vec (fill-pointer vec)) newel)
+    (setf (fill-pointer vec) (1+ (fill-pointer vec)))
+    (1- (fill-pointer vec))))
+
+(defun vector-push-extend (newel vec &optional (ext 1))
+  (if (>= (fill-pointer vec) (array-dimension vec 0))
+      (adjust-array vec (+ (array-dimension vec 0) ext)))
+  (vector-push newel vec))
+
+
 (defun string= (str1 str2 &key (start1 0) end1 (start2 0) end2)
   (setq str1 (string str1))
   (setq str2 (string str2))
@@ -1621,6 +1743,11 @@
   `(integer 0 ,(1- s)))
 
 
+(deftype extended-char ()
+  nil)
+
+
+
 (defun consp (obj)
   (typep obj 'cons))
 
@@ -1636,6 +1763,9 @@
 (defun keywordp (obj)
   (typep obj 'keyword))
 
+(defun compiled-function-p (obj)
+  (typep obj 'compiled-function))
+
 (defun functionp (obj)
   (typep obj 'function))
 
@@ -1709,6 +1839,12 @@
 
 
 
+(defmacro assert (test &optional places (conddat "Assertion does not hold") condargs)
+  `(unless ,test
+     (apply 'error ,conddat ,condargs)))
+
+
+
 (defun macroexpand (form)
   (do ((ret (list form t))
        expp)
@@ -1736,7 +1872,7 @@
     (t (eq x y))))
 
 
-(defun equalp (x y)
+#|(defun equalp (x y)
   (cond
     ((and (numberp x) (numberp y)) (= x y))
     ((and (characterp x) (characterp y)) (char-equal x y))
@@ -1756,7 +1892,7 @@
 				      (unless (equalp (row-major-aref x i) (row-major-aref y i))
 					(return-from equalp nil))))))
     (t (eq x y))))
-
+|#
 
 
 (defun fdefinition (fname)
@@ -1874,6 +2010,13 @@
 
 
 
+(deftype logical-pathname ()
+  'pathname)
+
+(defun logical-pathname (fn)
+  (pathname fn))
+
+
 (defun pathname-host (fn &key case))
 
 (defun pathname-device (fn &key case))
@@ -1885,13 +2028,36 @@
 (defun pathname-version (fn &key case))
 
 
+(defun namestring (fn) (pathname-name fn))
+
+(defun file-namestring (fn) (pathname-name fn))
+
+(defun directory-namestring (fn))
+
+(defun host-namestring (fn))
+
+(defun enough-namestring (fn &optional d) (pathname-name fn))
+
+
+(defun merge-pathnames (pn &optional defpn defv)
+  (pathname pn))
+
+
+
 (defun file-author (fn))
 
+(defun file-write-date (fn))
+
+(defun user-homedir-pathname (&optional h))
+
 
 
 (defmacro with-open-file ((str fn &rest opts) &body forms)
   `(let ((,str (open ,fn ,@opts)))
-     ,@forms))
+     (unwind-protect
+	  (progn
+	    ,@forms)
+       (close ,str))))
 
 
 
@@ -1930,6 +2096,26 @@
   obj)
 
 
+(defun write-to-string (obj &rest opts &key array base case circle escape gensym
+			      length level lines miser-width pprint-dispatch
+			      pretty radix readably right-margin)
+  (let ((out (make-string-output-stream)))
+    (apply 'write obj :stream out opts)
+    (get-output-stream-string out)))
+
+
+(defun prin1-to-string (obj)
+  (let ((out (make-string-output-stream)))
+    (prin1 obj out)
+    (get-output-stream-string out)))
+
+
+(defun princ-to-string (obj)
+  (let ((out (make-string-output-stream)))
+    (princ obj out)
+    (get-output-stream-string out)))
+
+
 (defmacro do-all-symbols (varres &body body)
   (let ((packsym (gensym)))
     `(progn
@@ -2022,16 +2208,14 @@
 	       (cond
 		 ((eq (car iter) :from)
 		  (setf (elt iter 6) (cadr f)))
-		 ((eq (car iter) :in)
-		  (setf (elt iter 4) (cadr f)))
-		 ((eq (car iter) :on)
-		  (setf (elt iter 3) (cadr f))))
+		 ((member (car iter) '(:in :on))
+		  (setf (elt iter 4) (cadr f))))
 	       (setq f (cddr f)))
 	      ((string= sym "IN")
 	       (setq iter `(:in ,(car iter) nil ,(cadr f) #'cdr))
 	       (setq f (cddr f)))
 	      ((string= sym "ON")
-	       (setq iter `(:on ,(car iter) ,(cadr f) #'cdr))
+	       (setq iter `(:on ,(car iter) nil ,(cadr f) #'cdr))
 	       (setq f (cddr f)))
 	      ((string= sym "=")
 	       (setq iter `(:eq ,(car iter) ,(cadr f) nil))
@@ -2039,6 +2223,9 @@
 	      ((string= sym "THEN")
 	       (setf (elt iter 3) (cadr f))
 	       (setq f (cddr f)))
+	      ((string= sym "ACROSS")
+	       (setq iter `(:across ,(car iter) nil nil ,(cadr f)))
+	       (setq f (cddr f)))
 	      ((string= sym "BEING")
 	       (setq iter `(:hashtable ,(car iter) nil nil nil))
 	       (setq f (cdr f))
@@ -2069,8 +2256,10 @@
 	  (progn
 	    (setq forms f)
 	    (return nil))))
-    (if (member (car iter) '(:from :in))
+    (if (member (car iter) '(:from :in :on :across))
 	(setf (elt iter 2) (gensym)))
+    (if (eq (car iter) :across)
+	(setf (elt iter 3) (gensym)))
     (when (eq (car iter) :hashtable)
       (unless (elt iter 1)
 	(setf (elt iter 1) (gensym)))
@@ -2161,19 +2350,38 @@
     (values out forms vars returnvar)))
 
 
+(defun flatten-tree-skipping-nils (tree)
+  (let (out)
+    (if (atom tree)
+	(list tree)
+	(do nil
+	    (nil)
+	  (if (null tree)
+	      (return-from flatten-tree-skipping-nils out))
+	  (if (or (atom tree) (car tree))
+	      (setq out (append (flatten-tree-skipping-nils (if (consp tree)
+								(car tree)
+								tree))
+				out)))
+	  (if (atom tree)
+	      (return-from flatten-tree-skipping-nils out))
+	  (setq tree (cdr tree))))))
+
+
 (defmacro loop (&body forms)
   (let (block-name
-	(tagname (gensym))
-	(middle-block-name (gensym))
+	(body-tag (gensym))
+	(prologue-tag (gensym))
 	(inner-block-name (gensym))
+	(skip-body-sym (gensym))
 	initially-forms
 	finally-forms
 	do-forms
 	iters
 	var vars parvars
 	lets l
-	outdestbinds od
-	indestbinds id
+	initial-setup iteration-setup
+	out o
 	returnvar)
     (do ()
 	((not forms))
@@ -2193,13 +2401,23 @@
 		 (when parvars
 		   (setq vars (cons parvars vars))
 		   (setq parvars nil))
-		 (setq var `(,(cadr forms) ,(cadddr forms)))
-		 (setq parvars (cons var parvars))
-		 (setq forms (cddddr forms)))
+		 (setq var `(,(cadr forms) nil))
+		 (if (and (symbolp (caddr forms))
+			  (string= (caddr forms) "="))
+		     (progn
+		       (setf (cadr var) (cadddr forms))
+		       (setq forms (cddddr forms)))
+		     (setq forms (cddr forms)))
+		 (setq parvars (cons var parvars)))
 		((string= sym "AND")
-		 (setq var `(,(cadr forms) ,(cadddr forms)))
-		 (setq parvars (cons var parvars))
-		 (setq forms (cddddr forms)))
+		 (setq var `(,(cadr forms) nil))
+		 (if (and (symbolp (caddr forms))
+			  (string= (caddr forms) "="))
+		     (progn
+		       (setf (cadr var) (cadddr forms))
+		       (setq forms (cddddr forms)))
+		     (setq forms (cddr forms)))
+		 (setq parvars (cons var parvars)))
 		((or (string= sym "IF")
 		     (string= sym "WHEN")
 		     (string= sym "UNLESS"))
@@ -2245,155 +2463,175 @@
     (if parvars
 	(setq vars (cons parvars vars)))
     (setq vars (reverse vars))
+
+    (setq lets `(let (,skip-body-sym) nil))
+    (setq l (cddr lets))
+
     (dolist (v vars)
-      (if lets
-	  (progn
-	    (setf (car l) `(let ,v nil))
-	    (setq l (cddar l)))
-	  (progn
-	    (setq lets `(let ,v nil))
-	    (setq l (cddr lets)))))
-    (dolist (i iters)
-      (let ((outdb (cond
-		     ((eq (car i) :hashtable)
-		      `(let (,(cadddr i))
-			 (maphash (lambda (k v) (setq ,(elt i 3) (cons (cons k v) ,(elt i 3)))) ,(elt i 4))
-			 nil))
-		     ((member (car i) '(:on :eq))
-		      `(destructuring-bind (,(cadr i))
-			   (list ,(caddr i))
-			 nil))
-		     (t
-		      `(destructuring-bind (,(caddr i))
-			   (list ,(cadddr i))
-			 nil))))
-	    (listcheck `(progn
-			  (if (endp ,(elt i 3))
-			      (return-from ,middle-block-name nil))
-			  nil))
-	    (middb (if (eq (car i) :hashtable)
-		       `(let ((,(cadr i) (caar ,(elt i 3)))
-			      (,(caddr i) (cdar ,(elt i 3))))
-			  nil nil)
-		       `(destructuring-bind (,(cadr i))
-			    ,(if (eq (car i) :from)
-				 `(list ,(cadddr i))
-				 `(list (car ,(cadddr i))))
-			  nil)))
-	    (indb (if (eq (car i) :hashtable)
-		      `(let ((,(caddr i) (cdar ,(elt i 3)))
-			     (,(cadr i) (caar ,(elt i 3))))
-			 nil
-			 nil)
-		      `(destructuring-bind (,(cadr i))
-			   ,(if (eq (car i) :from)
-				`(list ,(caddr i))
-				`(list (car ,(caddr i))))
-			 nil))))
-	(if outdestbinds
-	    (progn
-	      (setf (car od) outdb)
-	      (setq od (cdddar od))
-	      (when (member (car i) '(:from :in :hashtable))
-	      	(when (not (eq (car i) :from))
-		  (setf (car od) listcheck)
-		  (setq od (cddar od)))
-		(setf (car od) middb)
-		(setq od (cdddar od))
-		(setf (car id) indb)
-		(setq id (cdddar id))))
-	    (progn
-	      (setf outdestbinds outdb)
-	      (setq od (cdddr outdestbinds))
-	      (when (member (car i) '(:from :in :hashtable))
-		(when (not (eq (car i) :from))
-		  (setf (car od) listcheck)
-		  (setq od (cddar od)))
-		(setf (car od) middb)
-		(setq od (cdddar od))
-		(setf indestbinds indb)
-		(setq id (cdddr indestbinds)))))))
-    `(block ,block-name
-       ,(let ((inner-body
-	       `(progn
-		  (block ,middle-block-name
-		    ,(let ((pre-initially-forms-body
-			    `(progn
-			       ,@initially-forms
-			       (block ,inner-block-name
-				 (tagbody
-				    ,tagname
-				    (if (or ,@(mapcar (lambda (x) (cond
-								    ((eq (car x) :from)
-								     (if (elt x 4)
-									 (if (= (elt x 5) 1)
-									     `(> ,(elt x 2) ,(elt x 4))
-									     `(< ,(elt x 2) ,(elt x 4)))))
-								    ((eq (car x) :on)
-								     `(endp ,(elt x 1)))
-								    ((eq (car x) :in)
-								     `(endp ,(elt x 2)))
-								    ((eq (car x) :hashtable)
-								     `(endp ,(elt x 3)))))
-						      iters))
-					,(if returnvar
-					     `(return-from ,block-name ,returnvar)
-					     `(return-from ,inner-block-name nil)))
-				    ,(let ((post-initially-forms-body
-					    `(progn
-					       ,@do-forms
-					       ,@(mapcar (lambda (x) (cond
-								       ((eq (car x) :on)
-									`(setq ,(elt x 1) (funcall ,(elt x 3) ,(elt x 1))))
-								       ((eq (car x) :eq)
-									`(setq ,(elt x 1) ,(or (elt x 3) (elt x 2))))
-								       ((eq (car x) :from)
-									`(setq ,(elt x 2) (+ ,(elt x 1) ,(* (elt x 5) (elt x 6)))))
-								       ((eq (car x) :in)
-									`(setq ,(elt x 2) (funcall ,(elt x 4) ,(elt x 2))))
-								       ((eq (car x) :hashtable)
-									`(setq ,(elt x 3) (cdr ,(elt x 3))))))
-							 iters)
-					       (go ,tagname))))
-				       (if indestbinds
-					   (progn
-					     (setf (car id) post-initially-forms-body)
-					     indestbinds)
-					   post-initially-forms-body))))
-			       ,@finally-forms
-			       (return-from ,block-name nil))))
-		       (if outdestbinds
-			   (progn
-			     (setf (car od) pre-initially-forms-body)
-			     outdestbinds)
-			   pre-initially-forms-body)))
-		  ,@finally-forms)))
-	  (if lets
-	      (progn
-		(setf (car l) inner-body)
-		lets)
-	      inner-body))
-       nil)))
+      (setf (car l) `(let ,v nil))
+      (setq l (cddar l)))
+
+    (let (vrs)
+      (dolist (i iters)
+	(when (eq (car i) :in)
+	  (setq vrs (append (flatten-tree-skipping-nils (elt i 1)) (cons (elt i 2) vrs)))
+	  (setq initial-setup `((cl-user:al-loopy-setq ,(elt i 1) (car ,(elt i 3)))
+				(when (endp ,(elt i 2))
+				  (setq ,skip-body-sym t)
+				  (go ,prologue-tag))
+				(cl-user:al-loopy-setq ,(elt i 2) ,(elt i 3))
+				. ,initial-setup))
+	  (setq iteration-setup `((cl-user:al-loopy-setq ,(elt i 1) (car ,(elt i 2)))
+				  (if (endp ,(elt i 2)) (return-from ,inner-block-name nil))
+				  (cl-user:al-loopy-setq ,(elt i 2) (funcall ,(elt i 4) ,(elt i 2)))
+				  . ,iteration-setup)))
+	(when (eq (car i) :on)
+	  (setq vrs (append (flatten-tree-skipping-nils (elt i 1)) (cons (elt i 2) vrs)))
+	  (setq initial-setup `((when (endp ,(elt i 2))
+				  (setq ,skip-body-sym t)
+				  (go ,prologue-tag))
+				(cl-user:al-loopy-setq ,(elt i 1) ,(elt i 3))
+				(cl-user:al-loopy-setq ,(elt i 2) ,(elt i 3))
+				. ,initial-setup))
+	  (setq iteration-setup `((if (endp ,(elt i 2)) (return-from ,inner-block-name nil))
+				  (cl-user:al-loopy-setq ,(elt i 1) ,(elt i 2))
+				  (cl-user:al-loopy-setq ,(elt i 2) (funcall ,(elt i 4) ,(elt i 2)))
+				  . ,iteration-setup)))
+	(when (eq (car i) :from)
+	  (setq vrs (cons (elt i 1) vrs))
+	  (setq initial-setup `(,@(if (elt i 4)
+				      (list `(when ,(if (= (elt i 5) 1)
+							`(> ,(elt i 1) ,(elt i 4))
+							`(< ,(elt i 1) ,(elt i 4)))
+					       (setq ,skip-body-sym t)
+					       (go ,prologue-tag))))
+				  (setq ,(elt i 1) ,(elt i 3))
+				  . ,initial-setup))
+	  (setq iteration-setup `(,@(if (elt i 4)
+					(list `(if ,(if (= (elt i 5) 1)
+							`(> ,(elt i 1) ,(elt i 4))
+							`(< ,(elt i 1) ,(elt i 4)))
+						   (return-from ,inner-block-name nil))))
+				    (setq ,(elt i 1) (+ ,(elt i 1) ,(* (elt i 5) (elt i 6))))
+				    . ,iteration-setup)))
+	(when (eq (car i) :eq)
+	  (setq vrs (cons (elt i 1) vrs))
+	  (setq initial-setup `((cl-user:al-loopy-setq ,(elt i 1) ,(elt i 2))
+				. ,initial-setup))
+	  (setq iteration-setup `((cl-user:al-loopy-setq ,(elt i 1) ,(or (elt i 3) (elt i 2)))
+				  . ,iteration-setup)))
+	(when (eq (car i) :across)
+	  (setq vrs (list* (elt i 1) (elt i 2) (elt i 3) vrs))
+	  (setq initial-setup `((setq ,(elt i 1) (aref ,(elt i 3) 0))
+				(setq ,(elt i 2) 0)
+				(setq ,(elt i 3) ,(elt i 4))
+				. ,initial-setup))
+	  (setq iteration-setup `((setq ,(elt i 1) (aref ,(elt i 3) ,(elt i 2)))
+				  (if (>= ,(elt i 2) (length ,(elt i 3)))
+				      (return-from ,inner-block-name nil))
+				  (incf ,(elt i 2))
+				  . ,iteration-setup)))
+	(when (eq (car i) :hashtable)
+	  (setq initial-setup `((cl-user:al-loopy-setq ,(elt i 2) (cdar ,(elt i 3)))
+				(cl-user:al-loopy-setq ,(elt i 1) (caar ,(elt i 3)))
+				(when (endp ,(elt i 3))
+				  (setq ,skip-body-sym t)
+				  (go ,prologue-tag))
+				(setq ,(elt i 3)
+				      (let (o) (maphash (lambda (k v) (setq o (cons (cons k v) o))) ,(elt i 4)) o))
+				. ,initial-setup))
+	  (setq iteration-setup `((cl-user:al-loopy-setq ,(elt i 2) (cdar ,(elt i 3)))
+				  (cl-user:al-loopy-setq ,(elt i 1) (caar ,(elt i 3)))
+				  (if (endp ,(elt i 3)) (return-from ,inner-block-name nil))
+				  (setq ,(elt i 3) (cdr ,(elt i 3)))
+				  . ,iteration-setup))))
+
+      (when vrs
+	(setf (car l) `(let ,vrs nil))
+	(setq l (cddar l)))
+
+      (let* ((out `(block ,block-name
+		     nil))
+	     (o (cddr out)))
+	(setf (car o) lets)
+	(setq o l)
+	(setf (car o) `(tagbody
+			  (progn ,@(reverse initial-setup))
+			  ,prologue-tag
+			  ,@initially-forms
+			  (unless ,skip-body-sym
+			    (block ,inner-block-name
+			      (tagbody
+				 ,body-tag
+				 ,@do-forms
+				 (progn ,@(reverse iteration-setup))
+				 (go ,body-tag))))
+			  ,@finally-forms
+			  ,(if returnvar
+			       `(return-from ,block-name ,returnvar))))
+	out))))
 
 
 
 (defun format (out fstr &rest args)
-  (let (in-spec at-sign colon sign num dirargs)
-    (dotimes (i (length fstr))
+  (let ((*standard-output* (if (not out)
+			       (make-string-output-stream)
+			       (if (streamp out)
+				   out
+				   *standard-output*)))
+	in-spec at-sign colon sign num dirargs iterbegin otherargs skip-mode
+	(case-conv #'identity))
+    (do ((i 0 (+ 1 i)))
+	((= i (length fstr)))
       (let ((ch (elt fstr i)))
 	(if in-spec
 	    (cond
-	      ((char= ch #\@) (setq at-sign t))
-	      ((char= ch #\:) (setq colon t))
-	      ((char= ch #\,) (setq dirargs (cons num dirargs) num nil))
-	      ((char= ch #\~) (write-char #\~) (setq in-spec nil))
-	      ((char= ch #\%) (write-char #\newline) (setq in-spec nil))
-	      ((char= ch #\&) (fresh-line) (setq in-spec nil))
-	      ((char-equal ch #\s) (prin1 (car args)) (setq args (cdr args)) (setq in-spec nil))
-	      ((char-equal ch #\a) (princ (car args)) (setq args (cdr args)) (setq in-spec nil)))
+	      ((char= ch #\@) (setq at-sign t dirargs (cons num dirargs) num 0))
+	      ((char= ch #\:) (setq colon t dirargs (cons num dirargs) num 0))
+	      ((digit-char-p ch) (setq num (+ (* 10 num) (digit-char-p ch))))
+	      ((char= ch #\,) (setq dirargs (cons num dirargs) num 0))
+	      ((char= ch #\~) (unless skip-mode (write-char #\~)) (setq in-spec nil))
+	      ((char= ch #\%) (unless skip-mode (write-char #\newline)) (setq in-spec nil))
+	      ((char= ch #\&) (unless skip-mode (fresh-line)) (setq in-spec nil))
+	      ((char= ch #\() (setq case-conv (lambda (s) (if (or (stringp s)
+								  (symbolp s)
+								  (characterp s))
+							      (string-downcase (string s))
+							      s))
+				    in-spec nil))
+	      ((char= ch #\)) (setq case-conv #'identity in-spec nil))
+	      ((char= ch #\{) (setq iterbegin i in-spec nil)
+	       (if at-sign
+		   (setq otherargs nil)
+		   (setq otherargs args args (car args))))
+	      ((char= ch #\}) (if args
+				  (setq i iterbegin)
+				  (setq iterbegin nil args (cdr otherargs)))
+	       (setq in-spec nil skip-mode nil))
+	      ((char= ch #\^) (unless args (setq skip-mode t)) (setq in-spec nil))
+	      ((char-equal ch #\s) (unless skip-mode (prin1 (car args))) (setq args (cdr args)) (setq in-spec nil))
+	      ((char-equal ch #\a) (unless skip-mode (princ (funcall case-conv (car args)))) (setq args (cdr args)) (setq in-spec nil))
+	      ((find (char-downcase ch) "doxr")
+	       (setq dirargs (cons num dirargs))
+	       (let ((*print-base* (cond
+				     ((char-equal ch #\d) 10)
+				     ((char-equal ch #\o) 8)
+				     ((char-equal ch #\x) 16)
+				     ((char-equal ch #\r) (car dirargs))))
+		     (*print-escape* nil)
+		     (*print-radix* nil)
+		     (*print-readably* nil))
+		 (unless skip-mode
+		   (write (car args)))
+		 (setq args (cdr args))
+		 (setq in-spec nil)))
+	      (t (setq in-spec nil)))
 	    (if (char= ch #\~)
-		(setq in-spec t at-sign nil colon nil sign nil num nil dirargs nil)
-		(write-char ch)))))))
+		(setq in-spec t at-sign nil colon nil sign nil num 0 dirargs nil)
+		(unless skip-mode
+		  (write-char ch))))))
+    (if (not out)
+	(get-output-stream-string *standard-output*))))
 
 
 
@@ -2410,59 +2648,156 @@
 
 
 
+(defmacro with-standard-io-syntax (&body forms)
+  `(let ((*package* (find-package 'cl-user))
+	 (*print-array* t)
+	 (*print-base* 10)
+	 (*print-escape* t)
+	 (*print-readably* t)
+	 (*read-base* 10)
+	 (*read-default-float-format* 'single-float)
+	 (*read-suppress* nil))
+     ,@forms))
+
+
+
+(defmacro handler-case (expr &rest clauses)
+  (let ((blockname (gensym))
+	(valssym (gensym))
+	handlers
+	noerrfunc)
+    (mapcar (lambda (cl)
+	      (if (eq (car cl) :no-error)
+		  (setq noerrfunc `(lambda ,(cadr cl) ,@(cddr cl)))
+		  (setq handlers (cons `(,(car cl)
+					  (lambda ,(or (cadr cl) (list (gensym))) (return-from ,blockname ,@(cddr cl))))
+				       handlers))))
+	    clauses)
+    `(let (,valssym)
+       (block ,blockname
+	 (handler-bind
+	     ,(reverse handlers)
+	   (setq ,valssym (multiple-value-list ,expr)))
+	 ,(if noerrfunc
+	      `(apply ,noerrfunc ,valssym)
+	      `(values-list ,valssym))))))
+
+
+
+(defmacro restart-case (form &rest clauses)
+  (let (restarts
+	tail
+	(blockname (gensym))
+	(argssym (gensym))
+	tagsyms)
+    (mapcar (lambda (cl)
+	      (let ((tag (gensym)))
+		(setq tagsyms (cons tag tagsyms))
+		(setq restarts (cons `(,(car cl) (lambda (&rest args)
+						   (setq ,argssym args)
+						   (go ,tag))) restarts))
+		(setq tail (list* tag
+				  `(return-from ,blockname
+				     (apply #'(lambda ,(cadr cl) . ,(cddr cl)) ,argssym))
+				  tail))))
+	    clauses)
+    `(let (,argssym)
+       (block ,blockname
+	 (tagbody
+	    (restart-bind
+		,(reverse restarts)
+	      (return-from ,blockname ,form))
+	    ,@tail)))))
+
+
+
+(defmacro with-simple-restart ((restname frm &rest args) &rest forms)
+  `(restart-case
+       (progn ,@forms)
+     (,restname nil (values nil t))))
+
+
+
+(defun find-restart (id &optional cond)
+  (let ((rs (compute-restarts)))
+    (or (find id rs :key #'car)
+	(find id rs :key #'cdr))))
+
+
+
+(defun break (&optional form &rest args)
+  (with-simple-restart (continue "")
+    (invoke-debugger (make-condition 'simple-condition)))
+  nil)
+
+
+
 (defun abort (&optional cond)
   (invoke-restart 'abort))
 
 
 
+(defun continue (&optional cond)
+  (let ((r (find-restart 'continue)))
+    (if r (invoke-restart r))))
 
-(export '(*features* machine-instance machine-type machine-version
-	  short-site-name long-site-name *modules* provide require describe
-	  describe-object inspect lambda-parameters-limit call-arguments-limit
-	  multiple-values-limit array-rank-limit array-dimension-limit
-	  array-total-size-limit char-code-limit lambda-list-keywords identity
-	  *read-default-float-format* pi 1+ 1- minusp plusp abs zerop signum mod
-	  rem evenp oddp gcd lcm isqrt conjugate cis upgraded-complex-part-type
-	  logand logandc1 logandc2 logeqv lognand lognor logorc1 logorc2 logxor
-	  boole-1 boole-2 boole-andc1 boole-andc2 boole-and boole-c1 boole-c2
-	  boole-clr boole-eqv boole-ior boole-nand boole-nor boole-orc1
-	  boole-orc2 boole-set boole-xor boole *gensym-counter* gensym gentemp
-	  first second third fourth fifth sixth seventh eighth ninth tenth rest
-	  caar cadr cdar cddr caaar caadr cadar caddr cdaar cdadr cddar cdddr
-	  caaaar caaadr caadar caaddr cadaar cadadr caddar cadddr cdaaar cdaadr
-	  cdadar cdaddr cddaar cddadr cdddar cddddr make-list copy-alist
-	  copy-tree tree-equal sublis nsublis endp butlast nbutlast acons
-	  pairlis when unless incf decf and or cond otherwise case ccase ecase
-	  typecase ctypecase etypecase return multiple-value-bind prog prog*
-	  every some notany notevery member member-if member-if-not find find-if
-	  find-if-not assoc assoc-if assoc-if-not rassoc rassoc-if rassoc-if-not
-	  position position-if position-if-not count count-if count-if-not
-	  remove remove-if-not delete delete-if delete-if-not remove-duplicates
-	  delete-duplicates substitute substitute-if substitute-if-not
-	  nsubstitute nsubstitute-if nsubstitute-if-not subst subst-if
-	  subst-if-not nsubst nsubst-if nsubst-if-not nreverse revappend nreconc
-	  adjoin fill replace push pop set-difference nset-difference union
-	  nunion intersection nintersection set-exclusive-or nset-exclusive-or
-	  subsetp mismatch search sort stable-sort array-rank array-dimension
-	  array-total-size array-in-bounds-p upgraded-array-element-type
+
+
+
+(export '(*features* *default-pathname-defaults* machine-instance machine-type
+	  machine-version short-site-name long-site-name *modules* provide
+	  require describe describe-object inspect lambda-parameters-limit
+	  call-arguments-limit multiple-values-limit array-rank-limit
+	  array-dimension-limit array-total-size-limit char-code-limit
+	  lambda-list-keywords identity constantly *read-default-float-format*
+	  pi 1+ 1- minusp plusp abs zerop signum mod rem evenp oddp gcd lcm
+	  isqrt conjugate cis upgraded-complex-part-type logand logandc1
+	  logandc2 logeqv lognand lognor logorc1 logorc2 logxor boole-1 boole-2
+	  boole-andc1 boole-andc2 boole-and boole-c1 boole-c2 boole-clr
+	  boole-eqv boole-ior boole-nand boole-nor boole-orc1 boole-orc2
+	  boole-set boole-xor boole *gensym-counter* gensym gentemp first second
+	  third fourth fifth sixth seventh eighth ninth tenth rest caar cadr
+	  cdar cddr caaar caadr cadar caddr cdaar cdadr cddar cdddr caaaar
+	  caaadr caadar caaddr cadaar cadadr caddar cadddr cdaaar cdaadr cdadar
+	  cdaddr cddaar cddadr cdddar cddddr make-list copy-alist copy-tree
+	  tree-equal sublis nsublis endp butlast nbutlast acons pairlis defsetf
+	  when unless define-modify-macro incf decf and or cond otherwise case
+	  ccase ecase typecase ctypecase etypecase return multiple-value-bind
+	  prog prog* every some notany notevery member member-if member-if-not
+	  find find-if find-if-not assoc assoc-if assoc-if-not rassoc rassoc-if
+	  rassoc-if-not position position-if position-if-not count count-if
+	  count-if-not remove remove-if-not delete delete-if delete-if-not
+	  remove-duplicates delete-duplicates substitute substitute-if
+	  substitute-if-not nsubstitute nsubstitute-if nsubstitute-if-not subst
+	  subst-if subst-if-not nsubst nsubst-if nsubst-if-not nreverse
+	  revappend nreconc adjoin fill replace push pushnew pop set-difference
+	  nset-difference union nunion intersection nintersection
+	  set-exclusive-or nset-exclusive-or subsetp mismatch search sort
+	  stable-sort array-rank array-dimension array-total-size
+	  array-in-bounds-p array-element-type upgraded-array-element-type
 	  adjustable-array-p get get-properties remprop char schar bit sbit
-	  svref string= string/= string< string<= string> string>= string-equal
-	  string-not-equal string-lessp string-not-greaterp string-greaterp
-	  string-not-lessp char/= char< char<= char> char>= char-equal
-	  char-not-equal char-lessp char-not-greaterp char-greaterp
-	  char-not-lessp digit-char digit-char-p char-int string-upcase
-	  string-downcase string-capitalize nstring-upcase nstring-downcase
-	  nstring-capitalize string-left-trim string-right-trim string-trim
-	  defpackage signed-byte unsigned-byte consp listp symbolp keywordp
+	  svref vector-pop vector-push vector-push-extend string= string/=
+	  string< string<= string> string>= string-equal string-not-equal
+	  string-lessp string-not-greaterp string-greaterp string-not-lessp
+	  char/= char< char<= char> char>= char-equal char-not-equal char-lessp
+	  char-not-greaterp char-greaterp char-not-lessp digit-char digit-char-p
+	  char-int string-upcase string-downcase string-capitalize
+	  nstring-upcase nstring-downcase nstring-capitalize string-left-trim
+	  string-right-trim string-trim defpackage signed-byte unsigned-byte
+	  extended-char consp listp symbolp keywordp compiled-function-p
 	  functionp packagep integerp rationalp floatp complexp random-state-p
 	  characterp standard-char-p vectorp simple-vector-p arrayp sequencep
 	  stringp simple-string-p bit-vector-p simple-bit-vector-p hash-table-p
-	  pathnamep streamp realp numberp check-type macroexpand equal equalp
+	  pathnamep streamp realp numberp check-type assert macroexpand equal
 	  fdefinition complement mapc mapcan maplist mapl mapcon map-into reduce
-	  merge pathname-host pathname-device pathname-directory pathname-type
-	  pathname-version file-author with-open-file terpri write-line
-	  write-sequence prin1 princ print do-all-symbols loop format
-	  encode-universal-time abort))
+	  merge logical-pathname pathname-host pathname-device
+	  pathname-directory pathname-type pathname-version namestring
+	  file-namestring directory-namestring host-namestring enough-namestring
+	  merge-pathnames file-author file-write-date user-homedir-pathname
+	  with-open-file terpri write-line write-sequence prin1 princ print
+	  write-to-string prin1-to-string princ-to-string do-all-symbols loop
+	  format encode-universal-time with-standard-io-syntax handler-case
+	  restart-case with-simple-restart find-restart break abort continue))
 
 
 
